%global amd_version_major 2
%global btf_version_major 1
%global camd_version_major 2
%global ccolamd_version_major 2
%global cholmod_version_major 3
%global colamd_version_major 2
%global csparse_version_major 3
%global cxsparse_version_major 3
%global klu_version_major 1
%global ldl_version_major 2
%global rbio_version_major 2
%global spqr_version_major 2
%global SuiteSparse_config_major 5
%global umfpack_version_major 5

Summary:        A collection of sparse matrix libraries
Name:           suitesparse
Version:        5.13.0
Release:        5%{?dist}
License:        (LGPLv2+ or BSD) and LGPLv2+ and GPLv2+
URL:            http://faculty.cse.tamu.edu/davis/suitesparse.html
Source0:        https://github.com/DrTimothyAldenDavis/SuiteSparse/archive/refs/tags/v%{version}/%{name}-%{version}.tar.gz

BuildRequires:  make gcc gcc-c++
BuildRequires:  openblas-devel openblas-devel tbb-devel hardlink chrpath
Provides:       ufsparse = %{version}-%{release}

%description
suitesparse is a collection of libraries for computations involving sparse
matrices.  The package includes the following libraries:
  AMD                 approximate minimum degree ordering
  BTF                 permutation to block triangular form (beta)
  CAMD                constrained approximate minimum degree ordering
  COLAMD              column approximate minimum degree ordering
  CCOLAMD             constrained column approximate minimum degree ordering
  CHOLMOD             sparse Cholesky factorization
  CSparse             a concise sparse matrix package
  CXSparse            CSparse extended: complex matrix, int and long int support
  KLU                 sparse LU factorization, primarily for circuit simulation
  LDL                 a simple LDL factorization
  SQPR                a multithread, multifrontal, rank-revealing sparse QR
                      factorization method
  UMFPACK             sparse LU factorization
  SuiteSparse_config  configuration file for all the above packages.
  RBio                read/write files in Rutherford/Boeing format


%package devel
Summary:        Development headers for SuiteSparse
Requires:       %{name} = %{version}-%{release}
Provides:       ufsparse-devel = %{version}-%{release}

%description devel
The suitesparse-devel package contains files needed for developing
applications which use the suitesparse libraries.


%package static
Summary:        Static version of SuiteSparse libraries
Requires:       %{name}-devel = %{version}-%{release}
Provides:       ufsparse-static = %{version}-%{release}

%description static
The suitesparse-static package contains the statically linkable
version of the suitesparse libraries.


%package -n %{name}64
Summary:        A collection of sparse matrix libraries (ILP64 version)

%description -n %{name}64
The suitesparse collection compiled against an ILP64 BLAS library.


%package -n %{name}64-devel
Summary:        Development headers for SuiteSparse (ILP64 version)
Requires:       %{name}-devel = %{version}-%{release}
Requires:       %{name}64 = %{version}-%{release}

%description -n %{name}64-devel
The suitesparse64-devel package contains files needed for developing
applications which use the suitesparse libraries (ILP64 version).


%package -n %{name}64-static
Summary:        Static version of SuiteSparse libraries (ILP64 version)
Requires:       %{name}-devel = %{version}-%{release}

%description -n %{name}64-static
The suitesparse64-static package contains the statically linkable
version of the suitesparse libraries (ILP64 version).


%package -n %{name}64_
Summary:        A collection of sparse matrix libraries (ILP64 version)

%description -n %{name}64_
The suitesparse collection compiled against an ILP64 BLAS library.


%package -n %{name}64_-devel
Summary:        Development headers for SuiteSparse (ILP64 version)
Requires:       %{name}-devel = %{version}-%{release}
Requires:       %{name}64_ = %{version}-%{release}

%description -n %{name}64_-devel
The suitesparse64_-devel package contains files needed for developing
applications which use the suitesparse libraries (ILP64 version) compiled
against a BLAS library with the "64_" symbol name suffix (see openblas-*64_
packages).


%package -n %{name}64_-static
Summary:        Static version of SuiteSparse libraries (ILP64 version)
Requires:       %{name}-devel = %{version}-%{release}

%description -n %{name}64_-static
The suitesparse64_-static package contains the statically linkable
version of the suitesparse libraries (ILP64 version) compiled against a
BLAS library with the "64_" symbol name suffix (see openblas-*64_ packages).


%package doc
Summary:        Documentation files for SuiteSparse
BuildArch:      noarch
Requires:       %{name} = %{version}-%{release}

%description doc
This package contains documentation files for %{name}.


%prep
%autosetup -c
pushd SuiteSparse-%{version}
  rm -r metis*
  ln -s %{_includedir}/metis/*.h include/

  for fil in $(grep -Frl 'pragma ivdep' .); do
    sed -i.orig 's/pragma ivdep/pragma GCC ivdep/' $fil
    touch -r ${fil}.orig $fil
    rm -f ${fil}.orig
  done

  sed -i -e '/^  CF =/ s/ -O3 -fexceptions//' SuiteSparse_config/SuiteSparse_config.mk

  sed -i -e '/SO.*=/s/$(LIBRARY).so/$(LIBRARY)$(LIBRARY_SUFFIX).so/' \
         -e '/AR_TARGET *=/s/$(LIBRARY).a/$(LIBRARY)$(LIBRARY_SUFFIX).a/' SuiteSparse_config/SuiteSparse_config.mk
  sed -i -e 's/-l\(amd\|btf\|camd\|ccolamd\|cholmod\|colamd\|csparse\|cxsparse\|klu\|ldl\|rbio\|spqr\|suitesparseconfig\|umfpack\)/-l\1$(LIBRARY_SUFFIX)/g' \
    $(find -name Makefile\* -o -name \*.mk)
popd
cp -a SuiteSparse-%{version} SuiteSparse64-%{version}
cp -a SuiteSparse-%{version} SuiteSparse64_-%{version}


%build
export AUTOCC=no

for build in SuiteSparse SuiteSparse64 SuiteSparse64_
do
  pushd $build-%{version}
  mkdir -p Doc/{AMD,BTF,CAMD,CCOLAMD,CHOLMOD,COLAMD,KLU,LDL,UMFPACK,SPQR,RBio} Include
  %set_build_flags
  export CFLAGS="$CFLAGS -I%{_includedir}/metis"
  export LAPACK=""

  if [ $build = SuiteSparse64 ]
  then
     export CFLAGS="$CFLAGS -DBLAS64"
     export BLAS=-lopenblas64
     export LIBRARY_SUFFIX=64
  elif [ $build = SuiteSparse64_ ]
  then
     export CFLAGS="$CFLAGS -DBLAS64 -DSUN64"
     export BLAS=-lopenblas64_
     export LIBRARY_SUFFIX=64_
  else
     export BLAS=-lopenblas
  fi   
   
  pushd SuiteSparse_config
    %make_build CFLAGS="$CFLAGS" BLAS="$BLAS" LIBRARY_SUFFIX="$LIBRARY_SUFFIX"
    cp -p *.h ../Include
  popd

  pushd AMD
    pushd Lib
      %make_build CFLAGS="$CFLAGS" BLAS="$BLAS" LIBRARY_SUFFIX="$LIBRARY_SUFFIX"
    popd
    cp -p Include/*.h ../Include
    cp -p README.txt Doc/License.txt Doc/lesser.txt Doc/ChangeLog Doc/*.pdf ../Doc/AMD
  popd

  pushd BTF
    pushd Lib
      %make_build CFLAGS="$CFLAGS" BLAS="$BLAS" LIBRARY_SUFFIX="$LIBRARY_SUFFIX"
    popd
    cp -p Include/*.h ../Include
    cp -p README.txt Doc/* ../Doc/BTF
  popd

  pushd CAMD
    pushd Lib
      %make_build CFLAGS="$CFLAGS" BLAS="$BLAS" LIBRARY_SUFFIX="$LIBRARY_SUFFIX"
    popd
    cp -p Include/*.h ../Include
    cp -p README.txt Doc/ChangeLog Doc/License.txt Doc/*.pdf ../Doc/CAMD
  popd

  pushd CCOLAMD
    pushd Lib
      %make_build CFLAGS="$CFLAGS" BLAS="$BLAS" LIBRARY_SUFFIX="$LIBRARY_SUFFIX"
    popd
    cp -p Include/*.h ../Include
    cp -p README.txt Doc/* ../Doc/CCOLAMD
  popd

  pushd COLAMD
    pushd Lib
      %make_build CFLAGS="$CFLAGS" BLAS="$BLAS" LIBRARY_SUFFIX="$LIBRARY_SUFFIX"
    popd
    cp -p Include/*.h ../Include
    cp -p README.txt Doc/* ../Doc/COLAMD
  popd

  pushd CHOLMOD
    pushd Lib
      %make_build CFLAGS="$CFLAGS" BLAS="$BLAS" LIBRARY_SUFFIX="$LIBRARY_SUFFIX"
    popd
    cp -p Include/*.h ../Include
    cp -p README.txt Doc/*.pdf ../Doc/CHOLMOD
    cp -p Cholesky/lesser.txt ../Doc/CHOLMOD/Cholesky_License.txt
    cp -p Core/lesser.txt ../Doc/CHOLMOD/Core_License.txt
    cp -p MatrixOps/gpl.txt ../Doc/CHOLMOD/MatrixOps_License.txt
    cp -p Partition/lesser.txt ../Doc/CHOLMOD/Partition_License.txt
    cp -p Supernodal/gpl.txt ../Doc/CHOLMOD/Supernodal_License.txt
  popd

  pushd CXSparse
    pushd Lib
      %make_build CFLAGS="$CFLAGS" BLAS="$BLAS" LIBRARY_SUFFIX="$LIBRARY_SUFFIX"
    popd
    cp -p Include/cs.h ../Include
    mkdir ../Doc/CXSparse/
    cp -p Doc/* ../Doc/CXSparse
  popd

  pushd KLU
    pushd Lib
      %make_build CFLAGS="$CFLAGS" BLAS="$BLAS" LIBRARY_SUFFIX="$LIBRARY_SUFFIX"
    popd
    cp -p Include/*.h ../Include
    cp -p README.txt Doc/lesser.txt ../Doc/KLU
  popd

  pushd LDL
    pushd Lib
      %make_build CFLAGS="$CFLAGS" BLAS="$BLAS" LIBRARY_SUFFIX="$LIBRARY_SUFFIX"
    popd
    cp -p Include/*.h ../Include
    cp -p README.txt Doc/ChangeLog Doc/lesser.txt Doc/*.pdf ../Doc/LDL
  popd

  pushd UMFPACK
    pushd Lib
      %make_build CFLAGS="$CFLAGS" BLAS="$BLAS" LIBRARY_SUFFIX="$LIBRARY_SUFFIX"
    popd
    cp -p Include/*.h ../Include
    cp -p README.txt Doc/License.txt Doc/ChangeLog Doc/gpl.txt Doc/*.pdf ../Doc/UMFPACK
  popd

  pushd SPQR
    pushd Lib
      %make_build CFLAGS="$CFLAGS -DHAVE_TBB -DNPARTITION" TBB=-ltbb BLAS="$BLAS" LIBRARY_SUFFIX="$LIBRARY_SUFFIX"
    popd
    cp -p Include/*.h* ../Include
    cp -p README{,_SPQR}.txt
    cp -p README_SPQR.txt Doc/* ../Doc/SPQR
  popd

  pushd RBio
    pushd Lib
      %make_build CFLAGS="$CFLAGS" BLAS="$BLAS" LIBRARY_SUFFIX="$LIBRARY_SUFFIX"
    popd
    cp -p Include/*.h ../Include
    cp -p README.txt Doc/ChangeLog Doc/License.txt ../Doc/RBio
  popd

  popd
done

%install
mkdir -p %{buildroot}%{_libdir}
mkdir -p %{buildroot}%{_includedir}/%{name}
cp -a SuiteSparse-%{version}/Include/*.{h,hpp} %{buildroot}%{_includedir}/%{name}/
for build in SuiteSparse SuiteSparse64 SuiteSparse64_
do
  pushd $build-%{version}
    cp -a */Lib/*.a lib/*.so* %{buildroot}%{_libdir}/
    chmod 755 %{buildroot}%{_libdir}/*.so.*

    rm -rf Licenses
    mkdir Licenses
    find */ -iname lesser.txt -o -iname lesserv3.txt -o -iname license.txt -o \
        -iname gpl.txt -o -iname GPLv2.txt -o -iname license \
        -a -not -type d | while read f; do
            b="${f%%/*}"
            r="${f#$b}"
            x="$(echo "$r" | sed 's|/doc/|/|gi')"
            install -m0644 -D "$f" "./Licenses/$b/$x"
        done
    hardlink -cv Docs/ Licenses/
  popd
done

chrpath --delete %{buildroot}%{_libdir}/*.so*

%check
export AUTOCC=no
export CC=gcc
TESTDIRS="AMD CAMD CCOLAMD CHOLMOD COLAMD KLU LDL SPQR RBio UMFPACK"
TESTDIRS="$TESTDIRS CXSparse"
for build in SuiteSparse SuiteSparse64 SuiteSparse64_
do
  pushd $build-%{version}
    export CFLAGS="%{optflags} -I%{_includedir}/metis"
    export LAPACK=""
    if [ $build = SuiteSparse64 ]
    then
       export CFLAGS="$CFLAGS -DBLAS64"
       export BLAS=-lopenblas64
       export LIBRARY_SUFFIX=64
    elif [ $build = SuiteSparse64_ ]
    then
       export CFLAGS="$CFLAGS -DBLAS64 -DSUN64"
       export BLAS=-lopenblas64_
       export LIBRARY_SUFFIX=64_
    else
       export BLAS=-lopenblas
    fi   

    for d in $TESTDIRS ; do
        %make_build -C $d/Demo CFLAGS="$CFLAGS" LIB="%{?__global_ldflags} -lm -lrt" BLAS="$BLAS" LIBRARY_SUFFIX="$LIBRARY_SUFFIX" SPQR_CONFIG=-DHAVE_TBB TBB=-ltbb
    done
  popd
done

%files
%license SuiteSparse-%{version}/Licenses
%{_libdir}/libamd.so.%{amd_version_major}*
%{_libdir}/libbtf.so.%{btf_version_major}*
%{_libdir}/libcamd.so.%{camd_version_major}*
%{_libdir}/libccolamd.so.%{ccolamd_version_major}*
%{_libdir}/libcholmod.so.%{cholmod_version_major}*
%{_libdir}/libcolamd.so.%{colamd_version_major}*
%{_libdir}/libcxsparse.so.%{cxsparse_version_major}*
%{_libdir}/libklu.so.%{klu_version_major}*
%{_libdir}/libldl.so.%{ldl_version_major}*
%{_libdir}/librbio.so.%{rbio_version_major}*
%{_libdir}/libspqr.so.%{spqr_version_major}*
%{_libdir}/libsuitesparseconfig.so.%{SuiteSparse_config_major}*
%{_libdir}/libumfpack.so.%{umfpack_version_major}*

%files devel
%{_includedir}/%{name}
%{_libdir}/lib*.so
%exclude %{_libdir}/lib*64*.so

%files static
%{_libdir}/lib*.a
%exclude %{_libdir}/lib*64*.a

%files -n %{name}64
%license SuiteSparse64-%{version}/Licenses
%{_libdir}/libamd64.so.%{amd_version_major}*
%{_libdir}/libbtf64.so.%{btf_version_major}*
%{_libdir}/libcamd64.so.%{camd_version_major}*
%{_libdir}/libccolamd64.so.%{ccolamd_version_major}*
%{_libdir}/libcholmod64.so.%{cholmod_version_major}*
%{_libdir}/libcolamd64.so.%{colamd_version_major}*
%{_libdir}/libcxsparse64.so.%{cxsparse_version_major}*
%{_libdir}/libklu64.so.%{klu_version_major}*
%{_libdir}/libldl64.so.%{ldl_version_major}*
%{_libdir}/librbio64.so.%{rbio_version_major}*
%{_libdir}/libspqr64.so.%{spqr_version_major}*
%{_libdir}/libsuitesparseconfig64.so.%{SuiteSparse_config_major}*
%{_libdir}/libumfpack64.so.%{umfpack_version_major}*

%files -n %{name}64-devel
%{_libdir}/lib*64.so

%files -n %{name}64-static
%{_libdir}/lib*64.a

%files -n %{name}64_
%license SuiteSparse64_-%{version}/Licenses
%{_libdir}/libamd64_.so.%{amd_version_major}*
%{_libdir}/libbtf64_.so.%{btf_version_major}*
%{_libdir}/libcamd64_.so.%{camd_version_major}*
%{_libdir}/libccolamd64_.so.%{ccolamd_version_major}*
%{_libdir}/libcholmod64_.so.%{cholmod_version_major}*
%{_libdir}/libcolamd64_.so.%{colamd_version_major}*
%{_libdir}/libcxsparse64_.so.%{cxsparse_version_major}*
%{_libdir}/libklu64_.so.%{klu_version_major}*
%{_libdir}/libldl64_.so.%{ldl_version_major}*
%{_libdir}/librbio64_.so.%{rbio_version_major}*
%{_libdir}/libspqr64_.so.%{spqr_version_major}*
%{_libdir}/libsuitesparseconfig64_.so.%{SuiteSparse_config_major}*
%{_libdir}/libumfpack64_.so.%{umfpack_version_major}*

%files -n %{name}64_-devel
%{_libdir}/lib*64_.so

%files -n %{name}64_-static
%{_libdir}/lib*64_.a

%files doc
%doc SuiteSparse-%{version}/Doc/*

%changelog
* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 5.13.0-5
- Rebuilt for OpenCloudOS Stream 23.09

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 5.13.0-4
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 5.13.0-3
- Rebuilt for OpenCloudOS Stream 23

* Thu Dec 08 2022 Miaojun Dong <zoedong@tencent.com> - 5.13.0-2
- Fix mail errors in changelog

* Thu Dec 01 2022 Miaojun Dong <zoedong@tencent.com> - 5.13.0-1
- Initial build
